library(GenomicFeatures)
library(rtracklayer)
library(ChIPseeker)
library(clusterProfiler)
library(org.Mm.eg.db)
library(tidyverse)
library(RColorBrewer)
library(ggplot2)
library(data.table)
library(dplyr)
library(biomaRt)
ensembl <- useEnsembl(biomart = "genes")
datasets <- listDatasets(ensembl)
ensembl <- useDataset(dataset = "mmusculus_gene_ensembl", mart = ensembl)

## Make txdb for dunnart annotation file
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/")
smiCra_txdb <- makeTxDbFromGFF("Scras_dunnart_assem1.0_pb-ont-illsr_flyeassem_red-rd-scfitr2_pil2xwgs2_60chr2.gff")

## geneID conversion tables
df2 <- read.table("biomart_ensembl.txt", header=TRUE, sep="\t")
df3 <- read.table("bioDBnet_refseq2ensembl_H3K27ac.txt", header=TRUE, sep="\t")


#################################################### DUNNART ANNOTATION ##########################################################

## Anotate peak files
annotatePeaks <- function(peak, outFile, outFile1, GOenrich, kegg, bg){
  
  # Annotate peak file based on dunnart GFF
  peakAnno <- annotatePeak(peak, tssRegion = c(-3000, 3000), TxDb = smiCra_txdb)

  # Write annotation to file
  write.table(peakAnno, outFile, sep="\t", quote=F, row.names=F)
  
  peakAnnoDF <- as.data.frame(peakAnno, row.names = NULL)
  # Convert refseq IDs and geneIDs to devil ensembl IDs
  peakAnnoDF$ensemblgeneID <- df2$Gene.stable.ID[match(unlist(peakAnnoDF$geneId), df2$Gene.name)]
  peakAnnoDF$ensemblgeneID = replace(peakAnnoDF$ensemblgeneID,is.na(peakAnnoDF$ensemblgeneID),"-")
  peakAnnoDF$transcriptIdAltered <- gsub("\\..*","", peakAnnoDF$transcriptId)
  peakAnnoDF$refseqID <- df3$Ensembl.Gene.ID[match(unlist(peakAnnoDF$transcriptIdAltered), df3$RefSeq.mRNA.Accession)]
  peakAnnoDF$refseqID = replace(peakAnnoDF$refseqID,is.na(peakAnnoDF$refseqID),"-")
  peakAnnoDF$combined <- ifelse(peakAnnoDF$refseqID == "-", peakAnnoDF$ensemblgeneID, peakAnnoDF$refseqID)
  peakAnnoDF$refseqID[peakAnnoDF$refseqID == as.character("-")] <- NA
  
  write.table(peakAnnoDF, outFile2, sep="\t", quote=F, row.names=F)

  # Convert devil ensembl to mouse ensembl
  peakAnnoDF$mouseensembl <- df2$Mouse.gene.stable.ID[match(unlist(peakAnnoDF$combined), df2$Gene.stable.ID)]
  # Write annotation with converted IDs 
  peakAnnoDF <- peakAnnoDF[!is.na(peakAnnoDF$mouseensembl),]
  write.table(peakAnnoDF, outFile1, sep="\t", quote=F, row.names=F)

  bg = fread(bg, header = FALSE)
  bg = unlist(bg)

  # GO enrichment analysis
  GO <- enrichGO(gene = peakAnnoDF$mouseensembl,
                 keyType = "ENSEMBL",
                 OrgDb = org.Mm.eg.db,
                 ont = "BP",
                 universe = bg,
                 pAdjustMethod = "BH",
                 pvalueCutoff = 0.01,
                 readable = TRUE)
  write.table(GO, GOenrich, sep="\t", quote=F, row.names=F)
  
  ## convert mouse ensembl to entrezID
  suppressWarnings(gene.df <- bitr(geneID=peakAnnoDF$mouseensembl, fromType="ENSEMBL", toType="ENTREZID", OrgDb="org.Mm.eg.db" ))
  
  ## kegg analysis
  kk <- enrichKEGG(gene = gene.df$ENTREZID, organism = "mmu", pvalueCutoff=0.05)
  write.table(kk, kegg, sep="\t", quote=F, row.names=F)
}

### Consensus peaks
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/consensus/enhancers/")
annotatePeaks(bg = "/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/go_background_orthologousENSEMBL_biomart.txt", 
peak = "dunnart_enhancer_peaks.narrowPeak", outFile = "dunnart_enhancer_annotation.txt", outFile1 = "dunnart_enhancer_annotationConvertedIDs.txt", 
GOenrich = "enhancer_mm10GOenrich", kegg="enhancer_KEGG.txt", outFile2 = "dunnart_enhancer_annotationConvertedIDs_tdevil.txt")

setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/consensus/promoters/")
annotatePeaks(bg = "/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/go_background_orthologousENSEMBL_biomart.txt", 
peak = "dunnart.clustered.narrowPeak", outFile = "cluster1_promoter_annotation.txt", outFile1 = "cluster1_promoter_annotationConvertedIDs.txt" , 
GOenrich = "cluster1_mm10GOenrich.txt", kegg="cluster1_KEGG.txt", outFile2 = "dunnart_cluster1_annotationConvertedIDs_tdevil.txt")

#################################################### MOUSE ANNOTATION ##########################################################

## Anotate peak files
annotatePeaksmm10 <- function(peak, outFile){
  
  # Annotate peak file based on mouse ENSEMBL annotation
  peakAnno <- annotatePeak(peak, tssRegion = c(-3000, 3000), TxDb = ensembl)
  
  # Write annotation to file
  write.table(peakAnno, outFile, sep="\t", quote=F, row.names=F)
}

### Consensus peaks
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/consensus/enhancers/")
annotatePeaksmm10(peak = "E10.5_enhancer_peaks.narrowPeak", outFile = "E10.5_enhancer_annotation.txt")
annotatePeaksmm10(peak = "E11.5_enhancer_peaks.narrowPeak", outFile = "E11.5_enhancer_annotation.txt")
annotatePeaksmm10(peak = "E12.5_enhancer_peaks.narrowPeak", outFile = "E12.5_enhancer_annotation.txt")
annotatePeaksmm10(peak = "E13.5_enhancer_peaks.narrowPeak", outFile = "E13.5_enhancer_annotation.txt")
annotatePeaksmm10(peak = "E14.5_enhancer_peaks.narrowPeak", outFile = "E14.5_enhancer_annotation.txt")
annotatePeaksmm10(peak = "E15.5_enhancer_peaks.narrowPeak", outFile = "E15.5_enhancer_annotation.txt")

setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/consensus/promoters/")
annotatePeaksmm10(peak = "E10.5_promoter_peaks.narrowPeak", outFile = "E10.5_promoter_annotation.txt")
annotatePeaksmm10(peak = "E11.5_promoter_peaks.narrowPeak", outFile = "E11.5_promoter_annotation.txt")
annotatePeaksmm10(peak = "E12.5_promoter_peaks.narrowPeak", outFile = "E12.5_promoter_annotation.txt")
annotatePeaksmm10(peak = "E13.5_promoter_peaks.narrowPeak", outFile = "E13.5_promoter_annotation.txt")
annotatePeaksmm10(peak = "E14.5_promoter_peaks.narrowPeak", outFile = "E14.5_promoter_annotation.txt")
annotatePeaksmm10(peak = "E15.5_promoter_peaks.narrowPeak", outFile = "E15.5_promoter_annotation.txt")

########################################### CLUSTERED MOUSE PROMOTERS #####################################################

# peakList = "consensus/promoters"
# peakFiles = list.files(peakList, pattern= "promoter_peaks.narrowPeak", full.names=T)
# peakData = lapply(peakFiles, function(x) fread(x, header=FALSE, sep="\t", quote = "", na.strings=c("", "NA"))) # read in all files
# peakData = list(peakData[[1]],peakData[[2]], peakData[[3]], peakData[[4]], peakData[[5]], peakData[[6]], peakData[[7]])
# names(peakData) = c("dunnart", "E10.5", "E11.5", "E12.5", "E13.5", "E14.5", "E15.5")

# clusterList = "consensus/promoters/clustered"
# clusterFiles = list.files(clusterList, pattern= "_annotation.txt", full.names=T)
# clusterData = lapply(clusterFiles, function(x) fread(x, header=TRUE, sep="\t", quote = "", na.strings=c("", "NA"))) # read in all files
# clusterIDs = lapply(clusterData, function(x) x$V4 %>% as.data.frame())
# clusterIDs = rbind(clusterIDs[[1]], clusterIDs[[2]], clusterIDs[[3]], clusterIDs[[4]], clusterIDs[[5]], clusterIDs[[6]], clusterIDs[[7]])
# test = as.vector(unlist(clusterIDs))
# promoters = lapply(peakData, function(x) x[x$V4 %in% test,])    
# lapply(names(promoters), function(x) write.table(promoters[[x]], file=paste(x,"clustered","narrowPeak", sep="."), sep="\t", quote=FALSE, col.names=FALSE, row.names=FALSE))

# setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/")
# annotatePeaksmm10(peak = "E10.5.clustered.narrowPeak", outFile = "cluster1_E10.5_promoter_annotation.txt", GOenrich = "cluster1_E10.5_promoter_mm10GOenrich", kegg="cluster1_E10.5_promoter_KEGG.txt")
# annotatePeaksmm10(peak = "E11.5.clustered.narrowPeak", outFile = "cluster1_E11.5_promoter_annotation.txt", GOenrich = "cluster1_E11.5_promoter_mm10GOenrich", kegg="cluster1_E11.5_promoter_KEGG.txt")
# annotatePeaksmm10(peak = "E12.5.clustered.narrowPeak", outFile = "cluster1_E12.5_promoter_annotation.txt", GOenrich = "cluster1_E12.5_promoter_mm10GOenrich", kegg="cluster1_E12.5_promoter_KEGG.txt")
# annotatePeaksmm10(peak = "E13.5.clustered.narrowPeak", outFile = "cluster1_E13.5_promoter_annotation.txt", GOenrich = "cluster1_E13.5_promoter_mm10GOenrich", kegg="cluster1_E13.5_promoter_KEGG.txt")
# annotatePeaksmm10(peak = "E14.5.clustered.narrowPeak", outFile = "cluster1_E14.5_promoter_annotation.txt", GOenrich = "cluster1_E14.5_promoter_mm10GOenrich", kegg="cluster1_E14.5_promoter_KEGG.txt")
# annotatePeaksmm10(peak = "E15.5.clustered.narrowPeak", outFile = "cluster1_E15.5_promoter_annotation.txt", GOenrich = "cluster1_E15.5_promoter_mm10GOenrich", kegg="cluster1_E15.5_promoter_KEGG.txt")


  annot <- as.data.frame(peakAnno@anno)
  
  # GO enrichment analysis
  GO <- enrichGO(gene = annot$geneId,
                 keyType = "ENSEMBL",
                 #universe = enh$geneId,
                 OrgDb = org.Mm.eg.db,
                 ont = "BP",
                 pAdjustMethod = "BH",
                 pvalueCutoff = 0.01,
                 readable = TRUE)
  write.table(GO, GOenrich, sep="\t", quote=F, row.names=F)
  
  ## convert mouse ensembl to entrezID
  suppressWarnings(gene.df <- bitr(geneID=annot$geneId, fromType="ENSEMBL", toType="ENTREZID", OrgDb="org.Mm.eg.db" ))
  
  ## kegg analysis
  kk <- enrichKEGG(gene = gene.df$ENTREZID, 
  #universe = enh$geneId, 
  organism = "mmu", pvalueCutoff=0.01)
  write.table(kk, kegg, sep="\t", quote=F, row.names=F)