# input: GO list for dunnart and each stage of mouse for ALL peaks
# output: simplified GO clustered heatmaps for each stage/species with word cloud annotations

# spartan modules
## module load foss/2020b
## module load r/4.0.4

BiocManager::install("gridtext", force=TRUE)
BiocManager::install("stringi", force=TRUE)
library(stringi)
library(gridtext)
library(tools)
library(devtools)
install_github("jokergoo/ComplexHeatmap")
install_github("jokergoo/simplifyEnrichment")
library(GOSemSim)
library(ComplexHeatmap)
library(circlize)
library(tidyverse)
library(simplifyEnrichment)
library(clusterProfiler)
library(data.table)
library(enrichplot)
library(org.Mm.eg.db)
library(ChIPseeker)

mmGO = godata('org.Mm.eg.db', ont="BP") 

## directories with the annotation files for peaks
enh_dir = "consensus/enhancers/annotation"
pro_dir = "consensus/promoters/clustered"
plot_dir = "consensus/plots/"
# Background for calculating enrichment for dunnart 
bg=fread("go_background_orthologousENSEMBL_biomart.txt", header=FALSE, sep="\t", quote = "", na.strings=c("", "NA"))
bg = unique(bg)
# Get GO enrich data for all files
# Simplify GO data based on semantic similarity to form clusters of similar words into a word cloud
# Also include adjust pvalue for GO term for each stage/species

geneOntologyEnrichment = function(peakList,plot){
    peaks = list.files(peakList, pattern= ".txt", full.names=T) # create list of files in directory
    peaks = as.list(peaks)
    data = lapply(peaks, function(x) fread(x, header=TRUE, sep="\t", quote = "", na.strings=c("", "NA"))) # read in all files
    data = lapply(data, function(x) x=setnames(x, old="geneId", new="mouseensembl", skip_absent=TRUE) %>% as.data.table())

    genes = lapply(data, function(i) as.data.frame(i)$mouseensembl)
    names(genes) = c("dunnart","E10.5","E11.5","E12.5", "E13.5","E14.5", "E15.5")
    genes = lapply(genes, function(i) unique(i))
    # compare cluster GO enrichment
    go_cluster = simplify(setReadable(
      compareCluster(
        geneCluster = genes, 
        fun = enrichGO, 
        ont="BP",
        universe = bg, 
        keyType="ENSEMBL", 
        pvalueCutoff = 0.001, 
        OrgDb = org.Mm.eg.db),
      OrgDb = org.Mm.eg.db, 
      keyType="ENSEMBL"))

    ck = pairwise_termsim(go_cluster, method = "Wang", semData=mmGO)

    ## Dotplot
    pdf(paste0(plot_dir, dotplot, sep=''), width = 9, height = 5)
    print(dotplot(ck, showCategory = 5) + 
    scale_color_viridis() +
    theme(axis.text.x = element_text(angle = 45, hjust=1)))
    dev.off()

    categorys <- c("ossification", "mesenchyme development") 
    categorys <- c("ossification", "cartilage development")   
    categorys <- c("muscle tissue development", "cartilage development")
    categorys <- c("cell-cell signaling by wnt", "Wnt signaling pathway")
  
    print(cnetplot(ck, showCategory = categorys, 
            cex_label_gene = 2, cex_gene = 1,
            cex_category = 0.2) + scale_fill_manual(values = c( "#000000","#4477AA", "#66CCEE", "#228833"
            , "#CCBB44", "#EE6677", "#AA3377")))  

}
geneOntologyEnrichment(peakList = pro_dir, plot = "promoter_simplifyGO_heatmap.pdf")
geneOntologyEnrichment(peakList = enh_dir, plot = "enhancer_simplifyGO_heatmap.pdf")


# GO enrichment analysis
    dunnartGO <- enrichGO(gene = genes$dunnart,
                 keyType = "ENSEMBL",
                 OrgDb = org.Mm.eg.db,
                 ont = "BP",
                 universe = bg, # background is all ENSEMBL IDS that have an orthologous ENSEMBL gene in mouse
                 pAdjustMethod = "BH",
                 pvalueCutoff = 0.01,
                 readable = TRUE)
    
    mouseGO <- lapply(genes[-1], function(x) enrichGO(gene = x,
                 keyType = "ENSEMBL",
                 OrgDb = org.Mm.eg.db,
                 ont = "BP",
                 pAdjustMethod = "BH",
                 pvalueCutoff = 0.01,
                 readable = TRUE))
    
    # Combine GO dataframes into a list
    go_data = list(dunnartGO@result, mouseGO$E10.5@result, mouseGO$E11.5@result, 
                    mouseGO$E12.5@result, mouseGO$E13.5@result, mouseGO$E14.5@result, 
                    mouseGO$E15.5@result)

    ## Filter for adjusted pvalue
    go_data_filtered = lapply(go_data, function(x) x$ID[x$p.adjust < 0.001])
    names(go_data_filtered) = c( "dunnart","E10.5", "E11.5", "E12.5", "E13.5", "E14.5", "E15.5")

    # Cluster GO terms using binary cut
    pdf(plot)
    simplifyGOFromMultipleLists(go_data_filtered, db = org.Mm.eg.db,  measure = "Wang", method = "binary_cut")
    dev.off()   
